package net.sergeych.kiloparsec.adapter

import kotlinx.coroutines.channels.ReceiveChannel

/**
 * Multiplatform implementation of an internet address.
 * Notice to implementors. It must provide correct and effective [equals] and [hashCode].
 */
interface NetworkAddress {
    val host: String
    val port: Int
}

/**
 * Multiplatform datagram abstraction
 */
interface Datagram {
    /**
     * Received message
     */
    val message: UByteArray

    /**
     * Address from where the message was sent
     */
    val address: NetworkAddress

    /**
     * Send a datagram in response, e.g., to the [address].
     * This method is optimized per single per-datagram use. If you need to send many datagram, use [DatagramConnector].
     */
    suspend fun respondWith(message: UByteArray)
}

@OptIn(ExperimentalStdlibApi::class)
interface DatagramConnector: AutoCloseable {

    val incoming: ReceiveChannel<Datagram>
    suspend fun send(message: UByteArray, networkAddress: NetworkAddress)
    @Suppress("unused")
    suspend fun send(message: UByteArray, datagramAddress: String) {
        send(message, networkAddressOf(datagramAddress))
    }

    suspend fun send(message: UByteArray,host: String,port: Int) =
        send(message, NetworkAddress(host,port))
    override fun close()
}

expect fun networkAddressOf(address: String): NetworkAddress
expect fun NetworkAddress(host: String,port: Int): NetworkAddress
